package webagency;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import agency.*;
import javax.naming.*;
import java.rmi.*;
import javax.rmi.*;
import javax.ejb.*;
import java.util.*;

public class CreateCustomerServlet extends HttpServlet
{
  private Agency agency;

  public void init(){
     try {
      InitialContext ic = new InitialContext();
      Object lookup = ic.lookup("java:comp/env/ejb/Agency");
      AgencyHome home = (AgencyHome)PortableRemoteObject.narrow(lookup, AgencyHome.class);
      agency = home.create();
    }
    catch (NamingException ex) {
      System.err.println(ex);
    }
    catch (ClassCastException ex) {
      System.err.println(ex);
    }
    catch (CreateException ex) {
      System.err.println(ex);
    }
    catch (RemoteException ex) {
      System.err.println(ex);
    }
  }

  private boolean createCustomer (PrintWriter out, String login, String name, String email) {
    if (login==null || login.length()==0) {
      out.println ("No login name specified - please try again");
      return false;
    }     
    if (name==null || name.length()==0) {
      out.println ("Full name not specified - please try again");
      return false;
    }     
    if (email==null || email.length()==0) {
      out.println ("Email address not specified - please try again");
      return false;
    }     
    try {
        synchronized(agency) {
          agency.createCustomer(login,name,email);
        }
      out.println ("New customer "+login+" created successfully");
      return true;
    }
    catch (DuplicateException ex) {
      out.println("Login name \""+login+"\" already in use - choose another");
      return false;
    }
    catch (CreateException ex) {
      out.println("Internal create error: "+ex);
      return false;
    }
    catch (RemoteException ex) {
      out.println("Internal remote error: "+ex);
      return false;
    }
  }

  private void showForm (PrintWriter out) {
    out.println ("<P>Enter details in all field to create a new customer</P>");
    out.println ("<FORM><TABLE>");
    out.println ("<TR><TD>Login:</TD><TD><INPUT type='text' name='login'</TD></TR>");
    out.println ("<TR><TD>Full name:</TD><TD><INPUT type='text' name='name'</TD></TR>");
    out.println ("<TR><TD>E-mail:</TD><TD><INPUT type='text' name='email'</TD></TR>");
    out.println ("<TR><TD colspan='2'><INPUT type='submit' value='Create Customer'></TD></TR>");
    out.println ("<TR><TD colspan='2'><INPUT type='reset'></TD></TR>");
    out.println ("</TABLE></FORM>");
  }

  public void doGet(HttpServletRequest req, HttpServletResponse res)
         throws ServletException, IOException {
    try {
      String agencyName;
      String customer = null;

      synchronized(agency) {
                agencyName = agency.getAgencyName();
            }
            
      res.setContentType ("text/html");
      PrintWriter out = res.getWriter();

      // print out page
      out.println ("<HTML>");
      out.println ("<HEAD><TITLE>" + agencyName + " Create Customer</TITLE></HEAD>");
      out.println ("<BODY><FONT FACE=ARIAL COLOR=DARKBLUE");
      out.println ("<H1><FONT SIZE=+3>" + agencyName + " Create Customer</FONT></H1>");

      // check if new customer specified
      String login = req.getParameter("login");
      String name = req.getParameter("name");
      String email = req.getParameter("email");
      if (login!=null || name!=null || email!=null) {
        if (!createCustomer(out,login,name,email))
          showForm(out);
      }
      else
        showForm (out);

      // format & display
      out.println ("</FONT></BODY>");
      out.println ("</HTML>");
    }
    catch (RemoteException ex) {
      System.err.println(ex);
    }
  }
}
