package webagency;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import agency.*;
import javax.naming.*;
import java.rmi.*;
import javax.rmi.*;
import javax.ejb.*;

public class AgencyTableServlet extends HttpServlet
{
  private final String tables = "<OPTION>Applicant<OPTION>ApplicantSkill<OPTION>Customer<OPTION>Job<OPTION>JobSkill<OPTION>Location<OPTION>Matched<OPTION>Skill";
  private Agency agency;
  private ServletContext context;

  public void init(){
    context = getServletContext();
    try {
      InitialContext ic = new InitialContext();
      Object lookup = ic.lookup("java:comp/env/ejb/Agency");
      AgencyHome home = (AgencyHome)PortableRemoteObject.narrow(lookup, AgencyHome.class);
      agency = home.create();
    }
    catch (NamingException ex) {
      context.log("NamingException in AgencyTableServlet.init", ex);
    }
    catch (ClassCastException ex) {
      context.log("ClassCastException in AgencyTableServlet.init", ex);
    }
    catch (CreateException ex) {
      context.log("CreateException in AgencyTableServlet.init", ex);
    }
    catch (RemoteException ex) {
      context.log("RemoteException in AgencyTableServlet.init", ex);
    }
  }

  public void destroy () {
    context = null;
    agency = null;
  }

  private void outputTable (PrintWriter out, String tableName, HttpServletResponse res) throws RemoteException{

    java.util.List query; // list of String[], first row = column names
    synchronized(agency) {
        query = agency.select(tableName);
      }

    out.println ("<P><FONT SIZE=+1>Listing of " + tableName + " table</FONT></P>");
    out.println ("<TABLE BORDER=1 BORDERCOLOR=SILVER BGCOLOR=IVORY CELLPADDING=5><TR>");

    String[] headerRow = (String[])query.get(0);
    for (int i = 0; i < headerRow.length; i++) {
      out.println ("<TH ALIGN=LEFT>" + headerRow[i] + "</TH>");
    }
    out.println ("</TR>");

    for (int i = 1; i < query.size(); i++) {
      out.println ("<TR>");
      String[] row = (String[])query.get(i);
      for (int r = 0; r < row.length; r++) {
        out.println ("<TD>" + row[r] + "</TD>");
      }
      out.println ("</TR>");
    }
    out.println ("</TABLE>");
  }

  public void doGet(HttpServletRequest req, HttpServletResponse res)
         throws IOException {
    try {
      String agencyName;
      String tableName = null;
        synchronized(agency) {
                agencyName = agency.getAgencyName();
            }
            
      res.setContentType ("text/html");
      PrintWriter out = res.getWriter();

      // print out form
      out.println ("<HTML>");
      out.println ("<HEAD><TITLE>" + agencyName + " List Tables</TITLE></HEAD>");
      out.println ("<BODY><FONT FACE=ARIAL COLOR=DARKBLUE>");
      out.println ("<H1><FONT SIZE=+3>" + agencyName + " List Tables</FONT></H1>");

      tableName = req.getParameter("tableList");
      if (tableName != null) {
         outputTable(out, tableName, res);
      }
      out.println ("<P>Select a table from the list to display the contents</P>");
      out.println ("<FORM>");
      out.println ("<SELECT NAME=\"tableList\" SIZE=1>" + tables + "</SELECT>");
      out.println ("<INPUT TYPE=submit>");
      out.println ("</FORM>");

      out.println ("</FONT></BODY>");
      out.println ("</HTML>");
    }
    catch (RemoteException ex) {
      context.log ("RemoteException in AgencyTableServlet.doGet", ex);
      res.sendError (res.SC_INTERNAL_SERVER_ERROR);
    }
  }
}
